<?php

namespace Luticate\Utils;

class LuBusiness {
    /**
     * @var array
     */
    private $parameters = [];

    /**
     * @var LuDataAccess
     */
    protected $dataAccess = null;

    /**
     * @param array $parametersSource
     */
    public function __construct(array $parametersSource)
    {
        $this->parameters = $parametersSource;
    }

    /**
     * @param string $name
     * @param mixed  $default
     * @return mixed
     */
    protected function getParam($name, $default = null)
    {
        return array_key_exists($name, $this->parameters) ? $this->parameters[$name] : $default;
    }

    /**
     * @param string $name
     * @param mixed  $value
     */
    public function setParam($name, $value)
    {
        $this->parameters[$name] = $value;
    }

    /**
     * @param string[] $params
     * @return bool
     */
    public function hasParam(array $params)
    {
        foreach ($params as $p)
        {
            if (!array_key_exists($p, $this->parameters))
                return false;
        }
        return true;
    }

    /**
     * @param string $param
     * @param mixed $validate
     * @return string
     */
    public function checkParam($param, $validate = null)
    {
        $value = $this->getParam($param);
        if (is_null($value) || trim($value) == '')
            abort(400, 'Missing parameter: ' . $param);
        if ($validate != null) {
            if (is_callable($validate)) {
                $validate = $validate($value);
            }
            else {
                $validate = preg_match($validate, $value);
            }
            if (!$validate)
                abort(400, 'Bad parameter value: ' . $param);
        }
        return $value;
    }

    /**
     * @param string
     */
    public function unauthorized($reason = 'Unauthorized')
    {
        abort(401, $reason);
    }

    /**
     * @param string
     */
    public function notFound($reason = 'Resource not found')
    {
        abort(404, $reason);
    }

    /**
     * @param $param string
     * @param $values string[]
     * @return string
     */
    public function getParamInArray($param, $values)
    {
        $value = $this->getParam($param);
        if (is_null($value) || !in_array($value, $values))
            $value = $values[0];
        return $value;
    }

    /**
     * @param $param string
     * @param $default int
     * @return int
     */
    public function getParamInt($param, $default = 0)
    {
        $value = $this->getParam($param);
        if (!is_null($value) && is_numeric($value))
            return (int)$value;
        return $default;
    }

    /**
     * @param $page int
     * @param $perPage int
     * @param $column string
     * @param $order string
     * @param $query string
     * @return LuDbo[]
     */
    public function search($page, $perPage, $column, $order, $query = null)
    {
        return $this->dataAccess->search($page, $perPage, $column, $order, $query);
    }

    /**
     * @param $id int
     * @return LuDbo|null
     */
    public function getById($id)
    {
        $data = $this->dataAccess->getById($id);
        if (is_null($data))
            $this->notFound();
        return $data;
    }

    /**
     * @param $id int
     */
    public function deleteById($id)
    {
        $this->dataAccess->deleteById($id);
    }

    /**
     * @param $data LuDbo
     * @return int
     */
    public function addId($data)
    {
        return $this->dataAccess->addId($data);
    }

    /**
     * @param $data LuDbo
     */
    public function add($data)
    {
        $this->dataAccess->add($data);
    }

    /**
     * @param $id int
     * @param $data LuDbo
     * @return LuDbo|null
     */
    public function editById($id, $data)
    {
        return $this->dataAccess->editById($id, $data);
    }
}
